from ai_dashboard.tabs.data_report.blocks.base import *
from ai_dashboard.tabs.data_report.blocks.basic import *
from ai_dashboard.tabs.data_report.blocks.lists import *
from ai_dashboard.tabs.data_report.blocks.connected import *
from ai_dashboard.tabs.data_report.blocks.media import *
from ai_dashboard.tabs.data_report.blocks.charts import *
from ai_dashboard.tabs.data_report.blocks.layout import *
from ai_dashboard.tabs.data_report.blocks.marks import *
